"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const d3_array_1 = require("d3-array");
const d3_shape_1 = require("d3-shape");
const react_1 = __importStar(require("react"));
const hooks_1 = require("../hooks");
const utils_1 = require("../utils");
const Markers_1 = require("./Markers");
const markersComps = {
    circle: Markers_1.Circle,
    square: Markers_1.Square,
    triangle: Markers_1.Triangle,
};
function LineSeries(props) {
    const [id] = react_1.useState(() => `series-${utils_1.getNextId()}`);
    const { data, label, ...otherProps } = props;
    // Update plot context with data description
    const { dispatch } = hooks_1.useDispatchContext();
    react_1.useEffect(() => {
        const [xMin, xMax] = d3_array_1.extent(data.x);
        const [yMin, yMax] = d3_array_1.extent(data.y);
        dispatch({ type: 'newData', value: { id, xMin, xMax, yMin, yMax, label } });
        // Delete information on unmount
        return () => dispatch({ type: 'removeData', value: { id } });
    }, [data, label, dispatch, id]);
    // Render stateless plot component
    return react_1.default.createElement(LineSeriesRender, Object.assign({}, otherProps, { data: data, label: label }));
}
exports.default = LineSeries;
function LineSeriesRender({ data, lineStyle, label, displayMarker, markerShape = 'circle', markerSize = 3, }) {
    // Get scales from context
    const { xScale, yScale, colorScaler } = hooks_1.usePlotContext();
    // calculates the path to display
    const color = colorScaler(label);
    const [path, markers] = react_1.useMemo(() => {
        if ([xScale, yScale].includes(null) || data.x.length !== data.y.length) {
            return [null, null];
        }
        // Format data for line function generator
        let series = [];
        for (let index = 0; index < data.x.length; index++) {
            const x = xScale(data.x[index]);
            const y = yScale(data.y[index]);
            series.push({ x, y });
        }
        // Calculate line from D3
        const lineGenerator = d3_shape_1.line()
            .x((d) => d.x)
            .y((d) => d.y);
        // Show markers
        const Marker = markersComps[markerShape];
        const markers = !displayMarker
            ? null
            : series.map(({ x, y }, i) => (react_1.default.createElement(Marker
            // eslint-disable-next-line react/no-array-index-key
            , { 
                // eslint-disable-next-line react/no-array-index-key
                key: `markers-${i}`, x: x, y: y, size: markerSize, fill: color })));
        return [lineGenerator(series), markers];
    }, [xScale, yScale, color, data, displayMarker, markerSize, markerShape]);
    if ([xScale, yScale].includes(null))
        return null;
    // default style
    const style = {
        fontFamily: 'Arial, Helvetica, sans-serif',
        stroke: color,
        strokeWidth: 2,
        ...lineStyle,
    };
    return (react_1.default.createElement("g", null,
        markers,
        react_1.default.createElement("path", { style: style, d: path, fill: "none" })));
}
